/*
	BibInputDevice - i.e. Wii Remote or GameCube Controller

*/

#include <string.h>
#include <string>
#include "BibWiiInputDevice.h"


BibWiiInputDevice::BibWiiInputDevice ()
{
}

void BibWiiInputDevice::Initialize (GXRModeObj * vMode)
{
	nCurActionSet = 0;
	nNumActions = 0;

	for (int i = 0; i < ACTION_MAP_SIZE; i ++)
	{
		for (int j = 0; j < MAX_ACTION_SETS; j ++)
		{
			ActionMaps [j] [i].ButtonAction = 0;
			ActionMaps [j] [i].eController = CTRL_NONE;
		}
	}

	// Wii Remote Initialization
	WPAD_Init();
	WPAD_SetDataFormat(0, WPAD_FMT_BTNS_ACC_IR);
	WPAD_SetVRes(0, vMode->fbWidth, vMode->xfbHeight);

	// GameCube Controller Initialization
	PAD_Init ();
}

BibWiiInputDevice::~BibWiiInputDevice ()
{
}


void BibWiiInputDevice::ControlMotor (eControllerTypes eController, int pad, bool bIsOn)
{
	if((pad<WPAD_CHAN_0) || (pad>WPAD_MAX_WIIMOTES))
		return;
	
	if (eController == CTRL_WII_REMOTE)
		WPAD_Rumble(pad, bIsOn);
	else if (eController == CTRL_GAMECUBE)
		PAD_ControlMotor(pad, bIsOn ? PAD_MOTOR_RUMBLE : PAD_MOTOR_STOP);
}


int BibWiiInputDevice::DefineAction (SingleAction da [MAX_ACTION_SETS])
{
int i;

	if ((nNumActions < 0) || (nNumActions >= ACTION_MAP_SIZE))
		return (-1);

	for (i = 0; i < MAX_ACTION_SETS; i ++)
	{
		ActionMaps [i] [nNumActions] . ButtonAction = da [i].ButtonAction;
		ActionMaps [i] [nNumActions] . eController = da [i].eController;
	}
	
	nNumActions ++;
	
	return (nNumActions - 1);
}


#ifdef WII_INPUT_DEVICE_DEBUG_MESSAGES
char szWiiInputDeviceMsg1 [128];
char szWiiInputDeviceMsg2 [128];
char szWiiInputDeviceMsg3 [128];
char szWiiInputDeviceMsg4 [128];
#endif

void BibWiiInputDevice::ScanInputDevice (float fFrameTimeDelta)
{
	PAD_ScanPads ();
	
	WPAD_ReadPending(WPAD_CHAN_ALL, NULL);

#ifdef WII_INPUT_DEVICE_DEBUG_MESSAGES
int res;
u32 type;
WPADData *wd;
	
	res = WPAD_Probe(0, &type);
	// Find out general status.
	switch(res)
	{
		case WPAD_ERR_NO_CONTROLLER:
			sprintf(szWiiInputDeviceMsg1, "  Wiimote not connected\n");
			return;
		break;
		case WPAD_ERR_NOT_READY:
			sprintf(szWiiInputDeviceMsg1, "  Wiimote not ready\n");
			return;
		break;
		case WPAD_ERR_NONE:
			sprintf(szWiiInputDeviceMsg1, "  Wiimote ready\n");
		break;
		default:
			sprintf(szWiiInputDeviceMsg1, "  Unknown Wimote state %d\n",res);
			return;
		break;
 	}


	// Get Detailed readings
	if(res == WPAD_ERR_NONE)
	{
		wd = WPAD_Data(0);
			
		if(wd->ir.valid)
		{
			sprintf(szWiiInputDeviceMsg2, "  Cursor: %.02f,%.02f @ %.02f deg\n",wd->ir.x, wd->ir.y, wd->ir.angle);
		}
		else
		{
			sprintf(szWiiInputDeviceMsg2, "  No Cursor\n");
		}
		
		if(wd->ir.raw_valid)
		{
			sprintf(szWiiInputDeviceMsg3, "  Distance: %.02fm, Yaw: %.02f deg\n", wd->ir.z, wd->orient.yaw);
		}
		else
		{
			sprintf(szWiiInputDeviceMsg3, "  IR raw not valid.\n");
		}

		sprintf(szWiiInputDeviceMsg4, "  Accel: XYZ: %3d,%3d,%3d  Pitch: %.02f  Roll:  %.02f", wd->accel.x,wd->accel.y,wd->accel.z, wd->orient.pitch, wd->orient.roll);
	}
#endif

}

int BibWiiInputDevice::ActionDown (int nPadChan, int nActionNum)
{

	if ((nActionNum >=0) && (nActionNum < ACTION_MAP_SIZE))
	{
		switch (ActionMaps [nCurActionSet] [nActionNum] . eController)
		{
			case CTRL_WII_REMOTE:
			case CTRL_EXT_NUNCHUK:
			case CTRL_EXT_CLASSIC:
				return ((ActionMaps [nCurActionSet] [nActionNum] . ButtonAction & WPAD_ButtonsDown (nPadChan)) == ActionMaps [nCurActionSet] [nActionNum] . ButtonAction);
			break;
			
			case CTRL_GAMECUBE:
				return ((ActionMaps [nCurActionSet] [nActionNum] . ButtonAction & PAD_ButtonsDown (nPadChan)) == ActionMaps [nCurActionSet] [nActionNum] . ButtonAction);
			break;
			
			default:
				return (0);
			break;
		}
	}

	return (0);
}

int BibWiiInputDevice::ActionHeld (int nPadChan, int nActionNum)
{
	if ((nActionNum >=0) && (nActionNum < ACTION_MAP_SIZE))
	{
		switch (ActionMaps [nCurActionSet] [nActionNum] . eController)
		{
			case CTRL_WII_REMOTE:
			case CTRL_EXT_NUNCHUK:
			case CTRL_EXT_CLASSIC:
				return ((ActionMaps [nCurActionSet] [nActionNum] . ButtonAction & WPAD_ButtonsHeld (nPadChan)) == ActionMaps [nCurActionSet] [nActionNum] . ButtonAction);
			break;
			
			case CTRL_GAMECUBE:
				return ((ActionMaps [nCurActionSet] [nActionNum] . ButtonAction & PAD_ButtonsHeld (nPadChan)) == ActionMaps [nCurActionSet] [nActionNum] . ButtonAction);
			break;
			
			default:
				return (0);
			break;
		}
	}

	return (0);
}

int BibWiiInputDevice::ActionUp (int nPadChan, int nActionNum)
{
	if ((nActionNum >=0) && (nActionNum < ACTION_MAP_SIZE))
	{
		switch (ActionMaps [nCurActionSet] [nActionNum] . eController)
		{
			case CTRL_WII_REMOTE:
			case CTRL_EXT_NUNCHUK:
			case CTRL_EXT_CLASSIC:
				return ((ActionMaps [nCurActionSet] [nActionNum] . ButtonAction & WPAD_ButtonsUp (nPadChan)) == ActionMaps [nCurActionSet] [nActionNum] . ButtonAction);
			break;
			
			case CTRL_GAMECUBE:
				return ((ActionMaps [nCurActionSet] [nActionNum] . ButtonAction & PAD_ButtonsUp (nPadChan)) == ActionMaps [nCurActionSet] [nActionNum] . ButtonAction);
			break;
			
			default:
				return (0);
			break;
		}
	}

	return (0);
}


bool BibWiiInputDevice::IsPresent (eControllerTypes eController, int nPadChan)
{
int res;
u32 type;
PADStatus PadStatus[PAD_CHANMAX];

	if ((nPadChan < 0) || (nPadChan > 3))
		return (false);

	switch (eController)
	{
		case CTRL_WII_REMOTE:
			res = WPAD_Probe(nPadChan, &type);

			// Did we find this Wii Remote?
			if (res == WPAD_ERR_NONE)
				return (true);
			else
				return (false);
		break;
		
		case CTRL_EXT_NUNCHUK:
			res = WPAD_Probe(nPadChan, &type);

			// Did we find this Wii Remote?
			if (res != WPAD_ERR_NONE)
				return (false);

			// What is attached to the Wii remote?
			if (type == EXP_NUNCHUK)
				return (true);
			else
				return (false);
		break;
		
		case CTRL_EXT_CLASSIC:
			res = WPAD_Probe(nPadChan, &type);

			// Did we find this Wii Remote?
			if (res != WPAD_ERR_NONE)
				return (false);

			// What is attached to the Wii remote?
			if (type == EXP_CLASSIC)
				return (true);
			else
				return (false);
		break;
		
		case CTRL_GUITAR_HERO_3:
			res = WPAD_Probe(nPadChan, &type);

			// Did we find this Wii Remote?
			if (res != WPAD_ERR_NONE)
				return (false);

			// What is attached to the Wii remote?
			if (type == EXP_GUITAR_HERO_3)
				return (true);
			else
				return (false);
		break;
		
		case CTRL_GAMECUBE:
			PAD_Read(&PadStatus[0]);
			
			// Any errors reading the pad?
			if (PadStatus[nPadChan].err == PAD_ERR_NONE)
				return (true);
			else
				return (false);
		break;
		
		default:
			return (false);
		break;
	}
	
	return (false);
}

void BibWiiInputDevice::SetCurActionSet (int nActionSet)
{
	if ((nActionSet >=0) && (nActionSet < ACTION_MAP_SIZE))
	{
		nCurActionSet = nActionSet;
	}
}


int BibWiiInputDevice::GC_StickX (int nPad)
{
	if((nPad<PAD_CHAN0) || (nPad>PAD_CHAN3))
		return 0;
		
	return PAD_StickX(nPad);
}

int BibWiiInputDevice::GC_StickY (int nPad)
{
	if((nPad<PAD_CHAN0) || (nPad>PAD_CHAN3))
		return 0;
		
	return PAD_StickY(nPad);
}


bool BibWiiInputDevice::Wii_Orientation (int nPad, int & pitch, int & yaw, int & roll)
{
int res;
u32 type;
WPADData *wd;

	res = WPAD_Probe(nPad, &type);
	if (res != WPAD_ERR_NONE)
		return (false);

	wd = WPAD_Data(nPad);

	pitch = wd->orient.pitch;
	yaw = wd->orient.yaw;
	roll  = wd->orient.roll;

	return (true);
}


bool BibWiiInputDevice::Nunchuk_Stick (int nPad, int & angle, int & magnitude)
{
int res;
u32 type;
WPADData *wd;

	res = WPAD_Probe(nPad, &type);
	if (res != WPAD_ERR_NONE)
		return (false);

	if (type != EXP_NUNCHUK)
		return (false);

	wd = WPAD_Data(nPad);

	if (wd->exp.type != EXP_NUNCHUK)
		return (false);
	
	angle = wd->exp.nunchuk.js.ang;
	magnitude = wd->exp.nunchuk.js.mag * 100;

	return (true);
}

