/*

	SeekerEnemy Control Class

*/

#include <math.h>

#include "GameConstants.h"
#include "SeekerEnemyCtrl.h"
#include "ShooterWorld.h"

#include "Sprites/Explosion.sprite.h"
#include "Sprites/SeekerEnemy.sprite.h"


void SeekerEnemyCtrl::Initialize (class BibWorld * inpWorld)
{
	// Call the base class initialize.
	BibControl::Initialize (inpWorld);

	pWorld = (ShooterWorld *) inpWorld;

	// Add the sprites.
	basSpriteData . AddFrame (0, 100000, 0, SeekerEnemy_Bitmap, SEEKERENEMY_WIDTH, SEEKERENEMY_HEIGHT);
	basSpriteData . AddSequence ();
	basSpriteData . AddFrame (1, 30, 1, Explosion_Bitmap, EXPLOSION_WIDTH, EXPLOSION_HEIGHT);
	basSpriteData . AddFrame (1, 100000, 1, NULL, 0, 0);

	basSpriteData.SetCurSequence (0);
	basSpriteData.SetCurFrame (0);

	bpSize.x = basSpriteData.GetCurSprite () . GetWidth ();
	bpSize.y = basSpriteData.GetCurSprite () . GetHeight ();

	// This is a SeekerEnemy!
	nFlags = IS_ENEMY;

	bNewLock = true;
	fSeekSoundTimer = 0;
}

void SeekerEnemyCtrl::UnInitialize ()
{

}



bool SeekerEnemyCtrl::UpdateMovement (float fSpeedFactor)
{
BibPoint bpHeroLoc;
BibPointFP bpNextLoc;

	// If this was set outside of this function,
	//	then something (e.g. a bullet) wants us dead.
	if (SelfDestruct ())
	{
		ClearSelfDestruct ();
		basSpriteData.SetCurSequence (1);
	}

	// Call the base class to update sprite data, etc.
	if (BibControl::UpdateMovement (fSpeedFactor))
	{
		return (true);
	}

	// If we are in the exploding state, don't move.
	if (basSpriteData.GetCurSequence () == 1)
	{
		if (basSpriteData.GetCurFrame () == 1)
		{
			// We are done exploding, self destruct object.
			SetSelfDestruct ();
		}
		return (false);
	}


	/*
		Calculate the next position.
	*/

	// If we aren't on the screen, don't move.
	if (! pWorld ->IsInViewPort (bpMyLocation))
	{
		bNewLock = true;
		return (false);
	}

	if (bNewLock)
	{
		bNewLock = false;
		pWorld -> pSound . PlaySoundAsync (ShooterSound::SEEKER_LOCK);
		fSeekSoundTimer = 0;
	}
	else
	{
		fSeekSoundTimer += fSpeedFactor;
		if (fSeekSoundTimer > SEEKER_SOUND_TIME)
		{
			bNewLock = true;
		}
	}


	// This guy heads straight at the hero, if the hero is on the screen.
	bpHeroLoc = pWorld -> GetHeroCtrl () . GetLocation ();

	// Calculate the movement vector.
	bpNextLoc.x = bpHeroLoc.x - bpInternalLoc.x;
	bpNextLoc.y = bpHeroLoc.y - bpInternalLoc.y;

	// Normalize and set to the correct velocity.
	bpNextLoc.Normalize ();
	bpNextLoc = bpNextLoc * SEEKER_VELOCITY;

	// Apply vector to calculate next location.
	bpInternalLoc = bpInternalLoc + (bpNextLoc * fSpeedFactor);


	// Are we inside the map.
	if (bpInternalLoc.x < 0)
	{
		bpInternalLoc.x = 0;
	}
	else if (bpInternalLoc.x > MAP_SIZE_X - bpSize.x)
	{
		bpInternalLoc.x = MAP_SIZE_X - bpSize.x;
	}
	if (bpInternalLoc.y < 0)
	{
		bpInternalLoc.y = 0;
	}
	else if (bpInternalLoc.y > MAP_SIZE_Y - bpSize.y)
	{
		bpInternalLoc.y = MAP_SIZE_Y - bpSize.y;
	}

	// Set location for external access.
	bpMyLocation.x = (int) bpInternalLoc.x;
	bpMyLocation.y = (int) bpInternalLoc.y;


	return (false);
}



void SeekerEnemyCtrl::SetLocation (BibPoint & inbpLoc)
{
	BibControl::SetLocation (inbpLoc);
	bpInternalLoc.x = inbpLoc.x;
	bpInternalLoc.y = inbpLoc.y;
}
